package com.sksamuel.scoot.validation

import com.sksamuel.scoot.rest.Logging
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.beans.factory.config.AutowireCapableBeanFactory
import javax.validation.ConstraintValidatorFactory
import reflect.BeanProperty

/**
 * @author Stephen K Samuel
 *
 *         Delegates creation of a constraint validator to the bean factory, allowing one to use normal spring
 *         beans as constraint validators.
 *
 */
class DelegatingConstraintValidatingFactory extends ConstraintValidatorFactory with Logging {

    @BeanProperty
    @Autowired var beanFactory: AutowireCapableBeanFactory = _

    override def getInstance[T](key: Class[T]) = {

        try {
            beanFactory.getBean(key)
        } catch {
            case e: Exception => beanFactory.createBean(key)
        }
    }
}