package com.sksamuel.scoot.validation

import org.springframework.validation.{BindException, ObjectError, FieldError}
import collection.mutable.ArrayBuffer
import org.springframework.validation.Validator

/** @author Stephen Samuel */
class ValidationInfo {
    var errors: Array[ValidationError] = Array()
}

class ValidationError {

    var objectName: String = _
    var field: String = _
    var rejectedValue: String = _
    var codes: Array[String] = Array()
    var defaultMessage: String = _
}

class ValidationException(val validationErrors: Array[ValidationError]) extends RuntimeException

object ValidationUtil {

    def validate(validator: Validator, any: AnyRef, name: String) {
        val errors = new BindException(any, name)
        validator.validate(any, errors)
        if (errors.hasErrors) {
            throw new ValidationException(map(errors))
        }
    }

    def map(e: BindException) = {
        import scala.collection.JavaConverters._
        val validationErrors = new ArrayBuffer[ValidationError]()
        for (error <- e.getAllErrors.asScala) {
            validationErrors += translate(error)
        }
        validationErrors.toArray
    }

    def translate(validationError: ObjectError) = {
        val serializableValidationError = new ValidationError()

        serializableValidationError.objectName = validationError.getObjectName
        serializableValidationError.codes = validationError.getCodes
        //serializableValidationError.guments(validationError.getArguments())
        serializableValidationError.defaultMessage = validationError.getDefaultMessage

        if (validationError.isInstanceOf[FieldError]) {
            //            serializableValidationError.field(((FieldError) validationError).getField())
            //          serializableValidationError.setRejectedValue(((FieldError) validationError).getRejectedValue())
        }
        serializableValidationError
    }
}